//operation above map structures

#include <allegro.h>

using namespace std;

#include "cl_level.h"

void LEVEL::init()
{
	scrbuf = screen;
	obj_lowest_char = '-';
	mapbmp = backbmp = NULL;
	bin_map = back_map = NULL;
	texture_w = texture_h = 16;
	map_w = map_h = 0;
	scrbuf = screen;
	render_background = 1;
}

void LEVEL::create_map()
{
	destroy_map();
	int s = map_w * map_h * sizeof(int);

	if(s) {
		bin_map = (int *)malloc(s);
		bzero(bin_map,s);
		back_map = (int *)malloc(s);
		bzero(back_map,s);
		mapbmp = create_bitmap( map_w * texture_w, map_h * texture_h );
		backbmp = create_bitmap( map_w * texture_w, map_h * texture_h );
	}
}

void LEVEL::destroy_map()
{
	if(bin_map) { 
		free(bin_map);
		bin_map = NULL;
	}
	if(back_map) { 
		free(back_map);
		back_map = NULL;
	}
	destroy_bitmap_safe(mapbmp);
	destroy_bitmap_safe(backbmp);
}

int LEVEL::set_map_line(int i, string data)
{
	int f,g,dl;

	if((!map_w) || (!map_h)) {
		error("Map size is not defined, no modifications are allowed");
		return 3;
	}
	
	dl = (int)data.length();
	if(dl > map_w) {
		message("Too long map line %i (expected %i)",dl,map_w);
	}
	if(i>=map_h) {
		error("Setting map line - out of range!");
		return 1;
	}
	if(dl >= map_w) {
		g = map_w;
	} else {
		g = dl;
		message("Too small map line %i (expected %i)",g,map_w);
	}

	for(f = 0; f != g; f++) {
		bin_map[i*map_w + f] = data[f] - obj_lowest_char;
	}
	return 0;
}

void LEVEL::set_back_line(int i, string data)
{
	int f;

	if(i<map_h) {
		for(f = 0; f != map_w; f++) {
			back_map[i*map_w + f] = data[f] - obj_lowest_char;
		}
	} else {
		error("Setting back_map line - out of range!");
	}
}



void LEVEL::print_text_map()
{
	int i;

	cout<<"map="<<map_w<<","<<map_h<<endl;
	for(i = 0; i != map_w * map_h; i++) {
		cout << (char) (bin_map[i] + obj_lowest_char) ;
		if((i!=0) && ((i+1)%map_w == 0)) {
			cout << endl;
		}
	}
}

void LEVEL::print_text_bgmap()
{
	int i;

	cout<<"map="<<map_w<<","<<map_h<<endl;
	for(i = 0; i != map_w * map_h; i++) {
		cout << (char) (back_map[i] + obj_lowest_char) ;
		if((i!=0) && ((i+1)%map_w == 0)) {
			cout << endl;
		}
	}
}

void LEVEL::print_map(int x, int y)
{
	draw_sprite(scrbuf, disk->gettexture(bin_map[x+y*map_w]), 
			x*texture_w, y*texture_h);
}

void LEVEL::print_internal_map(int x, int y)
{
	blit(disk->gettexture(bin_map[x+y*map_w]), mapbmp,
			0, 0, x*texture_w, y*texture_h, 
			texture_w, texture_h);
	if(render_background) {
		blit(disk->gettexture(back_map[x+y*map_w]), backbmp,
				0, 0, x*texture_w, y*texture_h, 
				texture_w, texture_h);
	}

}

void LEVEL::render_map()
{
	int a,b;
	clear(mapbmp);

	if(render_background) { clear(backbmp); }
	
	for(a = 0; a != map_h; a++) {
		for(b = 0; b != map_w; b++) {
			print_internal_map(b,a);
		}
	}

	if(scrbuf) {
		masked_blit(mapbmp, backbmp, 0, 0, 0, 0, texture_w * map_w, 
			texture_h * map_h);
	} else {
		error("No backbuffer defined!");
	}

	//visual data is rendered on background bmp
	//data for checking were is walkable area is placed
	// in scrbuf
}


void LEVEL::print_map()
{
	canvas_x = texture_w * map_w;
	canvas_y = texture_h * map_h;
	
	if(scrbuf){
		if((canvas_x) && (canvas_y)) {
			blit(backbmp, scrbuf, 0, 0, 0, 0, texture_w * map_w, 
				texture_h * map_h);
		} else {
			error("Drawing map with zero size");
		}

//		cout<<scrbuf<<endl;
	} else {
		error("No screenbuffer defined!");
	}
}

int LEVEL::walkable(int x, int y)
{
	int i;

	if(mapbmp!=NULL) {
		i = getpixel(mapbmp, x, y);
	} else {
		return TEX_WALKABLE;
	}

	if(i == makecol16(255, 0, 255)) {
		return TEX_WALKABLE;
	} else {
		return TEX_SOLID;
	}
}

void LEVEL::bg_from_file(string name)
{
	string fname;
	fname = "first.dat#";
	fname += name;

	if(!backbmp) {
		destroy_bitmap(backbmp);
	}
	backbmp = load_tga(fname.c_str(), pal);
	if(backbmp == NULL) { error("Unable to find background image."); }
	render_background = 0;
}


void LEVEL::map_dimension(int x, int y)
{
	if((map_w != x) && (map_h!=y)) {
		map_w = x;
		map_h = y;
		create_map();
	}
}

void LEVEL::texture_dimension(int x, int y)
{
	if((texture_w != x) || (texture_h != y)) {
		texture_w = x;
		texture_h = y;
		create_map();
	}
}


void LEVEL::lowest_char(char c)
{
	obj_lowest_char = c;
}

char LEVEL::lowest_char()
{
	return obj_lowest_char;
}

void LEVEL::middle_map(deque<string> l) 
{
	int i, s = l.size();

	for(i=0; i!=s; i++) {
		set_map_line(i,l[i]); //treba napisat iterator poriadne
	}
}

void LEVEL::background_map(deque<string> l) 
{
	int i, s = l.size();

	for(i=0; i!=s; i++) {
		set_back_line(i,l[i]); //treba napisat iterator poriadne
	}
}


void LEVEL::render()
{
	render_map();
}


void LEVEL::set_datadisc(DATADISC *d)
{
	disk = d;
}

void LEVEL::set_textures(deque<BITMAP *> &txt)
{
}

void LEVEL::set_drawing(BITMAP *bmp)
{
	scrbuf = bmp;
}

void LEVEL::done()
{
	destroy_bitmap_safe(mapbmp,"level_map->mapbmp");
	destroy_bitmap_safe(backbmp,"level_map->backbmp");
}


void LEVEL::set_filename(string fname)
{
	filename = fname;
}

string LEVEL::get_filename()
{
	return filename;
}

