
#ifndef SIRIEL_DEFINE
#define SIRIEL_DEFINE

#define SCREEN_SIZE_X	640
#define SCREEN_SIZE_Y	480
#define COLOR_DEPTH	16
#define SLEEP_TIME	30000
#define MAX_LINE_LENGTH	1024
#define MAX_ANIM_BMP	64
#define TIME_STEP	1

#define BASIC_FREQUENCY	1000
#define BASIC_VOLUME	255
#define BASIC_PAN	127

#define COL_WHITE	makecol(255, 255, 255)
#define COL_GRAY5	makecol(220, 220, 220)
#define COL_GRAY4	makecol(200, 200, 200)
#define COL_GRAY3	makecol(120, 120 ,120)
#define COL_GRAY2	makecol(70, 70, 70)
#define COL_GRAY1	makecol(50, 50, 50)
#define COL_BLACK	0
#define COL_TRANS	makecol(255, 0, 255)

#define CONFIG_FILE	"siriel4.cfg"

#define STR_VERSION	"0.9"
#define STR_ENGINE	"Siriel4 engine"
#define STR_AUTHOR	"Author: Juraj Michalek, georgik@blucina.net\nhttp://georgik.blucina.net"
#define STR_URL		"http://siriel.sf.net"

//Screen modes
#define SCREEN_WINDOW	"window"
#define SCREEN_FULL	"full"

//Configuration defines
#define PARAM_DATAFILE	"datafile"
#define PARAM_LANGUAGE	"language"
#define PARAM_FONT	"font"
#define PARAM_HERO_ANIM	"hero_anim"
#define PARAM_SCREEN	"screen"

//Animation defines
#define PARAM_SIZEX	"sizex"
#define PARAM_SIZEY	"sizey"
#define PARAM_FILE	"file"

//Datadisc define
#define PARAM_TEXTURES	"textures"
#define PARAM_TEXTURE	"texture"
#define PARAM_AUTHOR	"author"
#define PARAM_ENGINE	"engine"
#define PARAM_NAME	"title"

//Level defines
#define PARAM_MAPX	"map_x_size"
#define PARAM_MAPY	"map_y_size"

//Default configuration defines
#define DEFAULT_DATAFILE	"main.dat"
#define DEFAULT_LANGUAGE	"english"
#define DEFAULT_FONT		"fntmain"
#define DEFAULT_HERO_ANIM	"txtsiriel"
#define DEFAULT_READ_MODE	"def"
#define DEFAULT_SCREEN		SCREEN_FULL
#define DEFAULT_WIDGET		"default"

//Default animation defines
#define DEFAULT_SIZEX	"16"
#define DEFAULT_SIZEY	"16"
#define DEFAULT_FILE	"dummy"

//Tag for animations
#define TAG_DEFINE	"def"
#define TAG_DATA	"data"
#define TAG_MAP		"map"

//Motion options
#define MOVE_NONE	0
#define MOVE_UP		1
#define MOVE_DOWN	2
#define MOVE_RIGHT	3
#define MOVE_LEFT	4
#define MOVE_JUMP	5
#define MOVE_JUMP_UP	6
#define MOVE_JUMP_RIGHT	7
#define MOVE_JUMP_LEFT	8
#define MOVE_PARTLY_LEFT	9
#define MOVE_PARTLY_RIGHT	10
#define MOVE_PARACHUTE		11

//Motion strings describing animation
#define MOVE_UP_STR		"up"
#define MOVE_DOWN_STR		"down"
#define MOVE_LEFT_STR		"left"
#define MOVE_RIGHT_STR		"right"
#define MOVE_JUMP_STR		"jump"
#define MOVE_JUMP_UP_STR	"jumpup"
#define MOVE_JUMP_RIGHT_STR	"jumpright"
#define MOVE_JUMP_LEFT_STR	"jumpleft"
#define MOVE_PARACHUTE_STR	"parachute"

//datadisc definition
#define DATADISC_DESCRIPTOR	"config"

//errors
#define ERROR_NO_DESC_FILE	-1
#define ERROR_NO_HASH_FILE	-1
#define NO_ERROR		0
#define ERROR_NO_IMG_FILE	1

//texture status
#define TEX_SOLID		0
#define TEX_WALKABLE		1

//object defaults
#define OBJ_NONE		""		//nothing

//objects types - Integers
#define OBJ_BORDER_X1		"border_x1"	//border of box
#define OBJ_BORDER_X2		"border_x2"	//border of box
#define OBJ_BORDER_Y1		"border_y1"	//border of box
#define OBJ_BORDER_Y2		"border_y2"	//border of box
#define OBJ_GROUP		"group" 
#define OBJ_ENERGY		"energy"	//energy of object
#define OBJ_KEY_LEFT		"key_left"
#define OBJ_KEY_RIGHT		"key_right"
#define OBJ_KEY_DOWN		"key_down"
#define OBJ_KEY_UP		"key_up"
#define OBJ_SCORE		"score"		//score of object
#define OBJ_SPEED_X		"speed_x"	//spped in x axis
#define OBJ_SPEED_Y		"speed_y"	//spped in y axis
#define OBJ_WEIGHT		"weight"	//weight of object, 0 = levitation

//object types - Bools
#define OBJ_ACTIVE		"active"	//object is active
#define OBJ_CONTROL		"control"	//object is controlled by keyborad
#define OBJ_DIRECTIONS		"directions"	//use different animations in object motion
#define OBJ_PICK		"pick"		//pick & give points

//object types - String
#define OBJ_COLLISION		"collision"	//call subroutine on collision
#define OBJ_PARACHUTE		"parachute"	//parachute animation name
#define OBJ_MOTION		"motion"	//name of motion logic
#define	OBJ_ROOM		"room"		//name of room where object exists



//motion logic
#define MOTION_NONE		""		//no motion
#define MOTION_SIMPLE		"simple"	//x & y axis motion using constants
#define MOTION_BOUNCE		"bounce"	//x & y axis motion with bouncing walls



//game
#define GAME_TYPE_ARCADE	1
#define GAME_TYPE_MAZE		2
#define GAME_OBJECT_GLOBAL	"global"	//object with this value is visible in all rooms
#define GAME_ROOM_MAIN		"main"		//main room of game

//Events
#define EVENT_NONE		"none"
#define EVENT_PICK		"pick"		//pick single object
#define EVENT_EXIT		"exit"		//exit from level
#define EVENT_COLLECTED		"collected"	//all object were collected
#define EVENT_LOOSE_LIFE	"loose_life"	//loose of life
#define EVENT_ENERGY_UP		"energy_up"	//increase of energy
#define EVENT_ENERGY_DOWN	"energy_down"	//decrease of energy
#define	EVENT_KEY_TAB		"key_tab"	//tabulator pressed
#define EVENT_KEY_ENTER		"key_enter"	//enter pressed

//EC = Event-code
#define EC_NONE			0
#define EC_REDRAW		1	//redraw screen
#define EC_EXPAND		2	//expand command line
#define EC_NOEXPAND		3	//disable cmd line expanding


//MENU
#define MENU_SECTOR_0		"0"
#define MENU_SECTOR_1		"1"
#define MENU_SECTOR_2		"2"
#define MENU_SECTOR_3		"3"
#define MENU_SECTOR_4		"4"
#define MENU_SECTOR_5		"5"
#define MENU_SECTOR_6		"6"
#define MENU_SECTOR_7		"7"
#define MENU_SECTOR_8		"8"


//TAGS
#define TAG_NEW_LINE		"<br>"

//WIDGET properties
#define WIDGET_TEXT_CENTER	"center"
#define WIDGET_LINE_SPACE	"line_space"


//Univers
#define UNIVERS_REALITY		1	//time of game
#define UNIVERS_CONSOLE		2	//time of console
#define UNIVERS_DIALOG		3	//time of widgets
#define UNIVERS_LOADING		4	//time of loading data
#define UNIVERS_COLAPS		5	//terminate univers


//Lua limits
#define LUA_MAX_BUF		64000
#define LUA_MAX_STACK		4096


#define ROOM_LAYERS		3	//number of layers in room

#endif

